.TH XCLE 3 "December 2004" Unix Libraries

.SH NAME

XCLE \- eXtensible Concatenative Language Engine

.SH DESCRIPTION

.B XCLE
provides a framework for software-based handling of executable code, combining 
easy generation and manipulation, with execution speed and memory efficiency at 
runtime. 

.B XCLE
implements most of the basic data types: integers, floats, strings, recursive 
lists and executable primitives, encapsulated in a generic object type. The API 
provides the means to integrate program building capabilities into software, 
handling both the data and code aspects of program generation and execution. The 
library as a whole provides the necessary framework for manipulating concatenative 
code. 

The object hierarchy tree is described below, showing object name, data, category 
(between brackets), and string representation (between quotes). 

.PD 0
.HP 0
.B
XCLE_Object
		Generic object
.B
			  [none]
.P
							---
.P
.I
	XCLE_Void
	Undefined object
.B
			  [null]
.P
							''
.P
.I
	XCLE_Intg
	Integer number on 32 bits
.B
	   [scal]
.P
							'123456'
.P
.I
	XCLE_Fltp
	Floating point number
.B
		  [scal]
.P
							'3.1415926e+00'
.P
.I
	XCLE_Strg
	Character string
.B
			  [scal]
.P
							'"any text"'
.P
.I
	XCLE_List
	All-purposes list
.B
			 [list]
.P
							'[ obj1 obj2 \.\.\. ]'
.P
.I
	XCLE_Code
	Dynamic executable code
.B
		[exec]
.P
							'<CODE_NAME>' or '<CODE_NAME:data>'
.PD

.TP
Note 1:
Objects in an XCLE_List can be separated by spaces, tabs, or linefeeds / carry returns

.TP
Note 2:
Here is a table of numeric values for special characters:
.PD 0
.P
		CHR	DEC	HEX			CHR	DEC	HEX
.P
		"	 34	0x22			:	 58	0x3A
.P
		<	 60	0x3C			>	 62	0x3E
.P
		[	 91	0x5B			]	 93	0x5D
.P
			172	0xAC
.PD



.SS Execution

"XCLE_ListEval" or "XCLE_ObjectEval" respectively take an XCLE_List or an XCLE_Object
in addition to an XCLE_Stack and XCLE_Hash, on which they will execute the List
or Object.

The XCLE_List type can contain any particular XCLE_Object. It also provides a mecanism
to build a program, since when a List gets executed, all its elements are executed in
turn.
All XCLE_Code executable instructions, as well as the execution of an XCLE_List,
use an XCLE_Stack and an XCLE_Hash. They take arguments and return results
on the XCLE_Stack, while having access to named variables in the XCLE_Hash.
Execution of an XCLE_List is the act to take every member of this list and deposit
it on the stack if it is an XCLE_Void, XCLE_Intg, XCLE_Strg, or XCLE_List, or
execute it if it is an XCLE_Code.
Execution stops when the end of the list is reached, or the execution of an
[exec] type failed, from lack of arguments, bad argument types, or some other
error.
These execution routines return an exception (an XCLE_Exception structure),
equal to XCLE_EXCEPTION_OK when the execution went well, or the raised
exception in case of error. The full exception stack can be found in the
execution context structure.


.SS Compiler

"XCLE_ListParse" or "XCLE_ObjectParse" respectively take a character string and produce
an XCLE_List or an XCLE_Object. The string is interpreted into objects so that printing
these objects would produce back the original string, or something very similar.



.SS Syntax

A valid string for "XCLE_ListParse" is one or more string representation of objects,
among:
.PD 0
.P
.I
	Instructions :
	e.g. ' <HELP> ', the code base building blocks
.P
.I
	Numbers   :
	e.g. ' 3.1425e+00 ', an integer or floating point number
.P
.I
	Strings   :
	e.g. ' "HELLO" ', a litteral string
.P
.I
	Lists     :
	e.g. ' [ obj1 obj2 ] ', a list of other objects
.PD


During parsing, a bareword is looked-up as a number, then if not applicable in
the set of available instructions, and finally parsed as a string if it could not be
found.



.SS Data

The memory representation of an 
.B XCLE_Object
has two parts: a generic memory and tracking structure, used for reference counting
and cross-referencing, and a content part, type-dependant.

The 
.B XCLE_Void
data type is the default type: it is only used to mark internal
.B XCLE_Object
use, or in a standard context that something has gone wrong in memory management.

The three scalar data types are simple counted memory segments: fixed size for the
.B XCLE_Intg
and 
.B XCLE_Fltp
types, variable size (which means keeping a
.I 'size'
register) for the
.B XCLE_Strg
type.

The 
.B XCLE_List
type is a variable-length vector, with buffer space before and after the section
 holding (in a consecutive manner) the XCLE_Object's. The buffer spaces enables
 fast insertion and deletion with the pop/push and shift/unshift operations.

As for the 
.B XCLE_Code
(executable) type, this is a complex structure, containing,
among other things, a pointer toward a segment of assembler code retaining the
actual implementation, input and output arity and type information, and a
formatted description.

The "system" types (
.B XCLE_Stack
and 
.B XCLE_Hash,
or named variables table) play an essential role in memory management: Objects
can be freed when no reference for them in one of these tables exist any more.
They also are the essential holders for instruction arguments and results.




.SH BUGS

Innumerable. Don't forget to report them, even if each bug correction is the source
for new ones...




.SH TODO

Implement pseudo-code (list-compiled code) file dumping, in a cross-platform format.




.SH AUTHOR

.B	Author   :
	Yann LANDRIN-SCHWEITZER
.P
.B	Contact  :
	varkhan@varkhan.net
.P
.B	Homepage :
	http://varkhan.varkhan.net/



.SH SEE ALSO

.I Container structures: 
.B XCLE_Containers(3)

.I Object types and methods: 
.B XCLE_Obj_generic(3), XCLE_Obj_void(3), XCLE_Obj_intg(3), XCLE_Obj_fltp(3), XCLE_Obj_strg(3), XCLE_Obj_list(3), XCLE_Obj_code(3)

.I Additionnal modules: 
.B XCLE_Mod_exec(3), XCLE_Mod_parse(3), XCLE_Mod_string(3), XCLE_Mod_write(3)


.SH OTHER TOOLS

.B XCLstd:
 a library defining a standard instrutions set

.B xcl:
 compiler/interpreter for the XCLE library

.B gxcl:
 a Gtk interface for interactively executing XCLE code and manipulating stack and variable lists.



.SH LICENSE

This library is free software;  you can redistribute it and/or modify it 
under  the terms of the  GNU Lesser General Public License  as published 
by the  Free Software Foundation;  either version 2.1 of the License, or 
(at your option) any later version. 


This  library  is  distributed  in  the hope that it will be useful, but 
WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser 
General Public License for more details.


You should have received a copy of the GNU Lesser General Public License 
along with this library;  if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 


