.TH XCLE_Containers 3 "2006-02-22" Unix Libraries



.SH NAME
.I Execution context structures - Instructions manual
.PD 0
.P
.PD
--------------------------------------------------


. XCLE:  eXtensible Concatenative Language Engine
. Copyright (C) 2000-2006 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
. 
. 
. This library is free software;  you can redistribute it and/or modify it
. under  the terms of the  GNU Lesser General Public License  as published
. by the  Free Software Foundation;  either version 2.1 of the License, or
. (at your option) any later version.
. 
. This  library  is  distributed  in  the hope that it will be useful, but
. WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
. MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
. General Public License for more details.
. 
. You should have received a copy of the GNU Lesser General Public License
. along with this library;  if not, write to the Free Software Foundation,
. Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
. 
. 
. Author:   Yann LANDRIN-SCHWEITZER
. Contact:  varkhan@varkhan.net
. Homepage: http://www.varkhan.net/




.SH XCLE_Stack definition



.PD 1
.P
.PD
.SS Function XCLE_StackAlloc
.PD 0


.I XCLE_Stack
.B XCLE_StackAlloc
(
.I unsigned
.I long
.B chksz
) ;

.PD 1
.P
.PD
Allocates a new, empty Stack.

.B
	Args
.RS
.IP _ 2
.I chksz
	: buffer size increment
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else the newly allocated Stack.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackFree
.PD 0


.I void
.B XCLE_StackFree
(
.I XCLE_Stack
.B stk
) ;

.PD 1
.P
.PD
Frees a Stack, dereferences and frees its contents.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack to free
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackDepth
.PD 0


.I unsigned
.I long
.B XCLE_StackDepth
(
.I XCLE_Stack
.B stk
) ;

.PD 1
.P
.PD
Returns the depth (number of elements) of a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack we want to get information about
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Stack was passed as argument
.IP "" 0
else the Stack's depth.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackPush
.PD 0


.I unsigned
.I long
.B XCLE_StackPush
(
.I XCLE_Stack
.B stk
,
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Pushes and references an object onto a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack to push on
.IP "" 0
.IP _ 2
.I obj
	: object to push
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
else the new depth of the Stack.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack or object was passed as argument
.IP "" 0
.IP _ 2
.I EACCES
	: if the Stack was freezed and consequently cannot receive objects
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackPop
.PD 0


.I XCLE_Object
.B XCLE_StackPop
(
.I XCLE_Stack
.B stk
) ;

.PD 1
.P
.PD
Pops and dereferences an object from a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: non-empty Stack
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if the Stack was NULL, empty or freezed
.IP "" 0
else the object popped from the stack.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack was passed as argument
.IP "" 0
.IP _ 2
.I EACCES
	: if the Stack was freezed and consequently cannot yield objects
.IP "" 0
.IP _ 2
.I ENOMEM
	: if a memory disallocation failed (non-fatal error)
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackGet
.PD 0


.I XCLE_Object
.B XCLE_StackGet
(
.I XCLE_Stack
.B stk
,
.I unsigned
.I long
.B num
) ;

.PD 1
.P
.PD
Gets an object at a specified level of a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: non-empty Stack
.IP "" 0
.IP _ 2
.I num
	: level at which to look for an object
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if the XCLE_Stack was NULL, or the level invalid
.IP "" 0
else the object at level 'num' of the Stack (first level is 1).
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL or empty Stack was passed as argument, or the level was invalid
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackPut
.PD 0


.I XCLE_Object
.B XCLE_StackPut
(
.I XCLE_Stack
.B stk
,
.I XCLE_Object
.B obj
,
.I unsigned
.I long
.B num
) ;

.PD 1
.P
.PD
Puts an XCLE_Object at a specified level of a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack to put in
.IP "" 0
.IP _ 2
.I obj
	: object to put
.IP "" 0
.IP _ 2
.I num
	: level at which to put an object
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if the Stack was NULL, or the level invalid
.IP "" 0
else the old object at the specified level of the Stack (first level is 1).
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack or object was passed as argument, or the level was invalid
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackMap
.PD 0


.I unsigned
.I long
.B XCLE_StackMap
(
.I XCLE_Stack
.B stk
,
.B 
,
.I void
.B *
.B dat
) ;

.PD 1
.P
.PD
Executes a map handler function on all objects of a Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack to map
.IP "" 0
.IP _ 2
.I obj
	: object to put
.IP "" 0
.IP _ 2
.I map
	: map handler
.IP "" 0
.IP _ 2
.I dat
	: user data pointer
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if the Stack or the map handler were NULL
.IP "" 0
.IP _ 2
.I -1
	: if the map handler retuned -1 at some time
.IP "" 0
else the sum of all the values returned by the map handler
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack or object was passed as argument, or the level was invalid
.IP "" 0
and any error generated by the map handler.
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The map handler takes several arguments:
.IP _ 2
.I lvl
	: the Stack level being currently mapped
.IP "" 0
.IP _ 2
.I obj
	: the Object at this level
.IP "" 0
.IP _ 2
.I dat
	: the user data pointer passed to XCLE_StackMap
.IP "" 0
The mapping stops when all the non-empty levels have been mapped, or the map handler
returns a value of ~0 (i.e. -1). 
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StackChkType
.PD 0


.I unsigned
.I long
.B XCLE_StackChkType
(
.I XCLE_Stack
.B stk
,
.I unsigned
.I long
.B argc
,
.I XCLE_Type
.B *
.B argt
) ;

.PD 1
.P
.PD
Checks the number and types of objects on an Stack.

.B
	Args
.RS
.IP _ 2
.I stk
	: Stack to type-check
.IP "" 0
.IP _ 2
.I argc
	: number of objects to check
.IP "" 0
.IP _ 2
.I argt
	: types of objects to check (or-ed types of admissible objects, level by level)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if the Stack was NULL or there were not enough objects on Stack
.IP "" 0
else an error flag (0 if all arguments were OK, every bit standing as an individual
error flag for each level).
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Stack was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD






.SH Variables domain definition



.PD 1
.P
.PD
.SS Function XCLE_HashAlloc
.PD 0


.I XCLE_Hash
.B XCLE_HashAlloc
(
.I unsigned
.I long
.B chksz
) ;

.PD 1
.P
.PD
Allocates an empty Hash.

.B
	Args
.RS
.IP _ 2
.I chksz
	: buffer size increment
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a new empty Hash.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashFree
.PD 0


.I void
.B XCLE_HashFree
(
.I XCLE_Hash
.B hsh
) ;

.PD 1
.P
.PD
Frees a Hash, dereferences and frees its contents.

.B
	Args
.RS
.IP _ 2
.I hsh
	: XCLE_Hash to free
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashGet
.PD 0


.I XCLE_Object
.B XCLE_HashGet
(
.I XCLE_Hash
.B hsh
,
.I char
.B *
.B name
) ;

.PD 1
.P
.PD
Gets an elements by its key in a Hash.

.B
	Args
.RS
.IP _ 2
.I hsh
	: Hash to query
.IP "" 0
.IP _ 2
.I name
	: key, or "Object name"
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Hash or name was passed as argument, or no object was indexed by this key
.IP "" 0
else the Object under this 'name'.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Hash or name was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashSet
.PD 0


.I XCLE_Object
.B XCLE_HashSet
(
.I XCLE_Hash
.B hsh
,
.I char
.B *
.B name
,
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Indexes an object under a key in a Hash, and reference it.

.B
	Args
.RS
.IP _ 2
.I hsh
	: Hash to query
.IP "" 0
.IP _ 2
.I name
	: key, or "Object name"
.IP "" 0
.IP _ 2
.I obj
	: object to set under its "name"
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if some error occurred
.IP "" 0
else the old object under this key, if it existed (and it is then dereferenced)
or the object passed as argument.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Hash, name or object was passed as argument
.IP "" 0
.IP _ 2
.I EACCES
	: if the Hash was freezed and consequently cannot receive objects
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashDel
.PD 0


.I XCLE_Object
.B XCLE_HashDel
(
.I XCLE_Hash
.B hsh
,
.I char
.B *
.B name
) ;

.PD 1
.P
.PD
Deletes a key and dereferences its contents in a Hash.

.B
	Args
.RS
.IP _ 2
.I hsh
	: Hash to query
.IP "" 0
.IP _ 2
.I name
	: key, or "Object name"
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Hash or name was passed as argument, or the key was not found
.IP "" 0
else the object under name 'name'.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Hash or name was passed as argument
.IP "" 0
.IP _ 2
.I EACCES
	: if the Hash was freezed and consequently cannot yield objects
.IP "" 0
.IP _ 2
.I ENOMEM
	: if a disallocation failed (non-fatal error)
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashMap
.PD 0


.I unsigned
.I long
.B XCLE_HashMap
(
.I XCLE_Hash
.B hsh
,
.B 
,
.I void
.B *
.B dat
) ;

.PD 1
.P
.PD
Executes a map handler function on all objects of a Hash.

.B
	Args
.RS
.IP _ 2
.I hsh
	: Hash to map
.IP "" 0
.IP _ 2
.I obj
	: object to put
.IP "" 0
.IP _ 2
.I map
	: map handler
.IP "" 0
.IP _ 2
.I dat
	: user data pointer
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if the Hash or the map handler were NULL
.IP "" 0
.IP _ 2
.I -1
	: if the map handler retuned -1 at some time
.IP "" 0
else the sum of all the values returned by the map handler
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Hash or object was passed as argument, or the level was invalid
.IP "" 0
and any error generated by the map handler.
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The map handler takes several arguments:
.IP _ 2
.I name
	: the Hash key being currently mapped
.IP "" 0
.IP _ 2
.I obj
	: the Object at this level
.IP "" 0
.IP _ 2
.I dat
	: the user data pointer passed to XCLE_HashMap
.IP "" 0
The mapping stops when all the keys have been mapped, or the map handler returns a
value of ~0 (i.e. -1). 
.RE
.PD 1
.P
.PD








