.TH XCLE_Mod_write 3 "2006-02-22" Unix Libraries



.SH NAME
.I Object Writing routines - Instructions manual
.PD 0
.P
.PD
---------------------------------------------


. XCLE:  eXtensible Concatenative Language Engine
. Copyright (C) 2000-2006 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
. 
. 
. This library is free software;  you can redistribute it and/or modify it
. under  the terms of the  GNU Lesser General Public License  as published
. by the  Free Software Foundation;  either version 2.1 of the License, or
. (at your option) any later version.
. 
. This  library  is  distributed  in  the hope that it will be useful, but
. WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
. MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
. General Public License for more details.
. 
. You should have received a copy of the GNU Lesser General Public License
. along with this library;  if not, write to the Free Software Foundation,
. Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
. 
. 
. Author:   Yann LANDRIN-SCHWEITZER
. Contact:  varkhan@varkhan.net
. Homepage: http://www.varkhan.net/




.SH Structures and typedefs



.PD 1
.P
.PD
.SS Typedef XCLE_WriteHandler
.PD 0
typedef unsigned long (* XCLE_WriteHandler) (void * dat, char * chr, unsigned long len) ;
.PD 1
.P
.PD
Handler for the write operation.

.B
	Note
.RS
The programmer has the responsability to keep in the 'dat' pointer
any useful information. The handler is charged to effect the actual
writing operation on whatever support is used, and must return the
number of characters actually written, that may be less than 'len'.
.RE
.PD 1
.P
.PD




.PD 1
.P
.PD
.SS Typedef XCLE_WriteFormat
.PD 0
typedef struct { ... } XCLE_WriteFormat ;
.PD 1
.P
.PD
Data formatting information for the XCLE_*Write methods.

.B
	Members
.RS
.IP _ 2
.I formflag
	: Format flags
.IP "" 0
.IP _ 2
.I intgprec
	: Precision (number of digits) for integers
.IP "" 0
.IP _ 2
.I intgbase
	: Base for printing integers
.IP "" 0
.IP _ 2
.I fltpprec
	: Precision (number of digits) for floating point numbers
.IP "" 0
.IP _ 2
.I fltpdeci
	: Decimals (digits after separ.) for floating point numbers
.IP "" 0
.RE
.PD 1
.P
.PD





.SH Writer routines for object types



.PD 1
.P
.PD
.SS Function XCLE_ObjectWrite
.PD 0


.I unsigned
.I long
.B XCLE_ObjectWrite
(
.I XCLE_Object
.B obj
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Object.

.B
	Args
.RS
.IP _ 2
.I obj
	: XCLE_Object to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Object was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_VoidWrite
.PD 0


.I unsigned
.I long
.B XCLE_VoidWrite
(
.I XCLE_Void
.B vd
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Void.

.B
	Args
.RS
.IP _ 2
.I vd
	: XCLE_Void to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Void was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_IntgWrite
.PD 0


.I unsigned
.I long
.B XCLE_IntgWrite
(
.I XCLE_Intg
.B in
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Intg.

.B
	Args
.RS
.IP _ 2
.I in
	: XCLE_Intg to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Intg was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_FltpWrite
.PD 0


.I unsigned
.I long
.B XCLE_FltpWrite
(
.I XCLE_Fltp
.B fp
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Fltp.

.B
	Args
.RS
.IP _ 2
.I fp
	: XCLE_Fltp to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Fltp was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_StrgWrite
.PD 0


.I unsigned
.I long
.B XCLE_StrgWrite
(
.I XCLE_Strg
.B str
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Strg.

.B
	Args
.RS
.IP _ 2
.I str
	: XCLE_Strg to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Strg was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListWrite
.PD 0


.I unsigned
.I long
.B XCLE_ListWrite
(
.I XCLE_List
.B lst
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_List.

.B
	Args
.RS
.IP _ 2
.I lst
	: XCLE_List to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_List was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeWrite
.PD 0


.I unsigned
.I long
.B XCLE_CodeWrite
(
.I XCLE_Code
.B cod
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
) ;

.PD 1
.P
.PD
Writing method for XCLE_Code.

.B
	Args
.RS
.IP _ 2
.I cod
	: XCLE_Code to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Code was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD






.SH Writer routines for execution context objects



.PD 1
.P
.PD
.SS Function XCLE_StackWrite
.PD 0


.I unsigned
.I long
.B XCLE_StackWrite
(
.I XCLE_Stack
.B stk
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
,
.I char
.B *
.B head
,
.I char
.B *
.B rowf
,
.I char
.B *
.B tail
,
.I unsigned
.I long
.B rown
) ;

.PD 1
.P
.PD
Writing method for XCLE_Code.

.B
	Args
.RS
.IP _ 2
.I stk
	: XCLE_Stack to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.IP _ 2
.I head
	: header line
.IP "" 0
.IP _ 2
.I rowf
	: row format line
.IP "" 0
.IP _ 2
.I tail
	: tailer line
.IP "" 0
.IP _ 2
.I rown
	: number of rows to write
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Stack, or an invalid format string, was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The row format string is printed for each stack row between 0 and rown-1.
Escape sequences take the form %pdd..dd?, where the 'p' char is an optionnal
padding character (by default ' '), 'd' chars are digits and '?' denotes either
the 'n' or 's' characters. They are replaced by the stack row number, for the 'n'
format specifier, and the corresponding object, for the 's' format, each written
on the number of chars denoted by the digits.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_HashWrite
.PD 0


.I unsigned
.I long
.B XCLE_HashWrite
(
.I XCLE_Stack
.B stk
,
.I XCLE_WriteHandler
.B wrt
,
.I void
.B *
.B dat
,
.I XCLE_WriteFormat
.B fmt
,
.I char
.B *
.B head
,
.I char
.B *
.B rowf
,
.I char
.B *
.B tail
) ;

.PD 1
.P
.PD
Writing method for XCLE_Code.

.B
	Args
.RS
.IP _ 2
.I stk
	: XCLE_Stack to write
.IP "" 0
.IP _ 2
.I wrt
	: handler for the real write operation
.IP "" 0
.IP _ 2
.I dat
	: user-set data pointer to pass the handler
.IP "" 0
.IP _ 2
.I fmt
	: data formatting information
.IP "" 0
.IP _ 2
.I head
	: header line
.IP "" 0
.IP _ 2
.I rowf
	: row format line
.IP "" 0
.IP _ 2
.I tail
	: tailer line
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if some error occurred
.IP "" 0
the total number of character needed, if 'wrt' is NULL
else the total number of characters printed
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL XCLE_Stack, or an invalid format string, was passed as argument
.IP "" 0
or any error generated by the 'wrt' handler.
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The row format string is printed for each key in the Hash.
Escape sequences take the form %pdd..dd?, where the 'p' char is an optionnal
padding character (by default ' '), 'd' chars are digits and '?' denotes either
the 'n' or 's' characters. They are replaced by the sh key, for the 'n' format
specifier, and the corresponding object, for the 's' format, each written on the
number of chars denoted by the digits.
.RE
.PD 1
.P
.PD







