.TH XCLE_Obj_code 3 "2006-02-22" Unix Libraries



.SH NAME
.I Code Object structure and methods - Instructions manual
.PD 0
.P
.PD
-------------------------------------------------------


. XCLE:  eXtensible Concatenative Language Engine
. Copyright (C) 2000-2006 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
. 
. 
. This library is free software;  you can redistribute it and/or modify it
. under  the terms of the  GNU Lesser General Public License  as published
. by the  Free Software Foundation;  either version 2.1 of the License, or
. (at your option) any later version.
. 
. This  library  is  distributed  in  the hope that it will be useful, but
. WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
. MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
. General Public License for more details.
. 
. You should have received a copy of the GNU Lesser General Public License
. along with this library;  if not, write to the Free Software Foundation,
. Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
. 
. 
. Author:   Yann LANDRIN-SCHWEITZER
. Contact:  varkhan@varkhan.net
. Homepage: http://www.varkhan.net/



.SH XCLE_Code memory management 



.PD 1
.P
.PD
.SS Function XCLE_CodeAlloc
.PD 0


.I XCLE_Code
.B XCLE_CodeAlloc
(
.B void
) ;

.PD 1
.P
.PD
Allocates a new Code object, with no execution handler nor data section.

.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a new Code object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This constructor is only there as a utility, for the returned code object is totally
useless: it has no name, no stack expectaion, no execution handler, and its data
section is undefined. Using it would only result in a place filler, and executing
it would do nothing.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeCopy
.PD 0


.I XCLE_Code
.B XCLE_CodeCopy
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Copies a Code object, along with its data segment.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code object to copy
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument or no memory was available
.IP "" 0
else a complete copy of the original Code object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This is a complete copy: the Code's data segment is also replicated.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeClone
.PD 0


.I XCLE_Code
.B XCLE_CodeClone
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Clones a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code object to clone
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument or no memory was available
.IP "" 0
else a simple copy of the Code object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This is a partial copy: the Code's data segment is the same as the original's.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeUpRef
.PD 0


.I XCLE_Code
.B XCLE_CodeDup
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Increases the reference count of a Code object and of its data segment.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to reference
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the referenced Code object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeDnRef
.PD 0


.I XCLE_Code
.B XCLE_CodeDnRef
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Decreases the reference count of a Code object and of its data segment.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to dereference
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the dereferenced Code object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeFree
.PD 0


.I void
.B XCLE_CodeFree
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Frees a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to free
.IP "" 0
.RE
.PD 1
.P
.PD






.SH XCLE_Code name, handler, and data



.PD 1
.P
.PD
.SS Function XCLE_CodeGetName
.PD 0


.I char
.B *
.B XCLE_CodeGetName
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Returns the name field of a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument, or no memory was available
.IP "" 0
else the name of this Code, as a \0-terminated string.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The returned char * buffer is allocated dynamically, with malloc. It is the
responsibility of the programmer to free it when it is not used anymore.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSetName
.PD 0


.I XCLE_Code
.B XCLE_CodeSetName
(
.I XCLE_Code
.B cod
,
.I char
.B *
.B name
) ;

.PD 1
.P
.PD
Updates the name field of a Code objetc.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to update
.IP "" 0
.IP _ 2
.I name
	: the new name (\0-terminated string)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument, or no memory was available
.IP "" 0
else the original 'cod' object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeGetHandler
.PD 0


.I void
.B *
.B XCLE_CodeGetHandler
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Returns a pointer to the execution handler of a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the pointer to the execution handler.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If you do not know what is the Code's execution handler, avoid this function.
For instruction set programmers: this is the same address as the 'func' member
in the CodeDef structure, that is, a CodeOperator function pointer.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSetHandler
.PD 0


.I XCLE_Code
.B XCLE_CodeSetHandler
(
.I XCLE_Code
.B cod
,
.I void
.B *
.B func
) ;

.PD 1
.P
.PD
Sets the  execution handler of a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to update
.IP "" 0
.IP _ 2
.I func
	: the new execution handler
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the original 'cod' object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If you do not know what is the Code's execution handler, avoid this function.
For instruction set programmers: execution handlers are generally set through
the parsing routines, that will look up a CodeDef structure from declared tables
and set execution handlers accordingly. Setting it by hand should be done with care.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeGetData
.PD 0


.I XCLE_Object
.B XCLE_CodeGetData
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Returns the data segment of a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the data segment.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If you do not know what is the Code's data segment, avoid this function.
For instruction set programmers: instructions have generally empty (NULL)
data segment. These are only used for parametrable instructions.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSetData
.PD 0


.I XCLE_Code
.B XCLE_CodeSetData
(
.I XCLE_Code
.B cod
,
.I XCLE_Object
.B data
) ;

.PD 1
.P
.PD
Sets the data segment of a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to update
.IP "" 0
.IP _ 2
.I data
	: data segment
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the original 'cod' objetc.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If you do not know what is the Code's data segment, avoid this function.
For instruction set programmers: you will probably let the users determines
data segments for parametrable instructions dynamically, through the parsing
routines. However, executing a checking pass on data segments before execution
can be useful. Combine the GetData and SetData methods with recursive ListMap
calls to provide a data segment verification pass.
.RE
.PD 1
.P
.PD






.SH XCLE_Code signature and prototypes



.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureNum
.PD 0


.I unsigned
.I short
.B XCLE_CodeSignatureNum
(
.I XCLE_Code
.B cod
) ;

.PD 1
.P
.PD
Returns the number of distinct signatures for this Code.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Code was passed as argument
.IP "" 0
else the number of distinct signatures.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
A Code's signature is a set of expected arguments number/types, and the corresponding
returned object number/types. Several can exist for a single Code, covering different
instruction semantics. This function only counts the number of such semantics.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureArgc
.PD 0


.I unsigned
.I short
.B XCLE_CodeSignatureArgc
(
.I XCLE_Code
.B cod
,
.I unsigned
.I short
.B sign
) ;

.PD 1
.P
.PD
Returns the number of arguments required for a particular signature.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.IP _ 2
.I sign
	: index of signature
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Code was passed as argument
.IP "" 0
else the number of arguments required for the signature of index 'sign'.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument, or the signature index was invalid
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
A Code's signature is a set of expected arguments number/types, and the corresponding
returned object number/types. Several can exist for a single Code, covering different
instruction semantics. This function will return the number of arguments expected for
the designated set.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureRetc
.PD 0


.I unsigned
.I short
.B XCLE_CodeSignatureRetc
(
.I XCLE_Code
.B cod
,
.I unsigned
.I short
.B sign
) ;

.PD 1
.P
.PD
Returns the number of results returned for a particular signature.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.IP _ 2
.I sign
	: index of signature
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Code was passed as argument
.IP "" 0
else the number of results returned for the signature of index 'sign'.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument, or the signature index was invalid
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
A Code's signature is a set of expected arguments number/types, and the corresponding
returned object number/types. Several can exist for a single Code, covering different
instruction semantics. This function will return the number of results returned for
the designated set.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureMatch
.PD 0


.I unsigned
.I short
.B XCLE_CodeSignatureMatch
(
.I XCLE_Code
.B cod
,
.I unsigned
.I short
.B argc
,
.I XCLE_Type
.B *
.B argt
,
.I unsigned
.I short
.B retc
,
.I XCLE_Type
.B *
.B rett
) ;

.PD 1
.P
.PD
Returns the number of signature matches for a given set of argument types and expected results.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to query
.IP "" 0
.IP _ 2
.I argc
	: number of arguments
.IP "" 0
.IP _ 2
.I argt
	: argument types (this is an XCLE_Type[argc] vector)
.IP "" 0
.IP _ 2
.I retc
	: number of expected returns
.IP "" 0
.IP _ 2
.I rett
	: types of expected returns (this is an XCLE_Type[retc] vector)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Code was passed as argument
.IP "" 0
else the number of signature matches.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This function computes the number of execution semantics supported by the Code object
and consistent with the given argument types and expected returns. In most cases,
nothing is expected of returned values, so setting 'retc' to 0 and 'rett' to NULL is
appropriate.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureAdd
.PD 0


.I XCLE_Code
.B XCLE_CodeSignatureAdd
(
.I XCLE_Code
.B cod
,
.I unsigned
.I short
.B argc
,
.I XCLE_Type
.B *
.B argt
,
.I unsigned
.I short
.B retc
,
.I XCLE_Type
.B *
.B rett
) ;

.PD 1
.P
.PD
Adds a new accepted signature for a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: the Code to update
.IP "" 0
.IP _ 2
.I argc
	: number of arguments
.IP "" 0
.IP _ 2
.I argt
	: argument types (this is an XCLE_Type[argc] vector)
.IP "" 0
.IP _ 2
.I retc
	: number of returns
.IP "" 0
.IP _ 2
.I rett
	: returns types (this is an XCLE_Type[retc] vector)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument or an error occured
.IP "" 0
else the updated 'cod' object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I ENOMEM
	: if a reallocation failed
.IP "" 0
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
A newly allocated Code object has no registered signature (XCLE_CodeSignatureNum would
return 0). This is the method to be used to set the signature(s) before profiling or
execution of objects using this Code.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeSignatureDel
.PD 0


.I XCLE_Code
.B XCLE_CodeSignatureDel
(
.I XCLE_Code
.B cod
,
.I unsigned
.I short
.B argc
,
.I XCLE_Type
.B *
.B argt
,
.I unsigned
.I short
.B retc
,
.I XCLE_Type
.B *
.B rett
) ;

.PD 1
.P
.PD
Deletes a signature from a Code object.

.B
	Args
.RS
.IP _ 2
.I cod
	: the Code to update
.IP "" 0
.IP _ 2
.I argc
	: number of arguments
.IP "" 0
.IP _ 2
.I argt
	: argument types (this is an XCLE_Type[argc] vector)
.IP "" 0
.IP _ 2
.I retc
	: number of returns
.IP "" 0
.IP _ 2
.I rett
	: returns types (this is an XCLE_Type[retc] vector)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL Code was passed as argument
.IP "" 0
else the updated 'cod' object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This method is provided for the sake of complteness, as deleting code signatures has
no real use. The 'argc', 'argt', 'retc' and 'rett' must have the same values as those
passed to XCLE_CodeSignatureAdd to add the targeted signature.
.RE
.PD 1
.P
.PD






.SH XCLE_Code object conversion and comparison



.PD 1
.P
.PD
.SS Function XCLE_ObjectToCode
.PD 0


.I XCLE_Code
.B XCLE_ObjectToCode
(
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Converts a generic object into a Code object, if possible.

.B
	Args
.RS
.IP _ 2
.I obj
	: Object to convert
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL object, or one with incompatible types, was passed as argument
.IP "" 0
else the actual Code object embedded into 'obj'.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL or invalid object was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_CodeEqual
.PD 0


.I unsigned
.I char
.B XCLE_CodeEqual
(
.I XCLE_Code
.B cod
,
.I XCLE_Code
.B codw
) ;

.PD 1
.P
.PD
Compares two Code objects.

.B
	Args
.RS
.IP _ 2
.I cod
	: Code to compare
.IP "" 0
.IP _ 2
.I codw
	: reference Code
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL Code was passed as argument
.IP "" 0
.IP _ 2
.I 0
	: if the two Codes were distinct
.IP "" 0
.IP _ 2
.I +1
	: if the two Codes had the same name, argument and return expectations, and data segment
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL Code was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD








