.TH XCLE_Obj_list 3 "2006-02-22" Unix Libraries



.SH NAME
.I List Object structure and methods - Instructions manual
.PD 0
.P
.PD
-------------------------------------------------------


. XCLE:  eXtensible Concatenative Language Engine
. Copyright (C) 2000-2006 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
. 
. 
. This library is free software;  you can redistribute it and/or modify it
. under  the terms of the  GNU Lesser General Public License  as published
. by the  Free Software Foundation;  either version 2.1 of the License, or
. (at your option) any later version.
. 
. This  library  is  distributed  in  the hope that it will be useful, but
. WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
. MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
. General Public License for more details.
. 
. You should have received a copy of the GNU Lesser General Public License
. along with this library;  if not, write to the Free Software Foundation,
. Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
. 
. 
. Author:   Yann LANDRIN-SCHWEITZER
. Contact:  varkhan@varkhan.net
. Homepage: http://www.varkhan.net/




.SH XCLE_List object type



.PD 1
.P
.PD
.SS Function XCLE_ListAlloc
.PD 0


.I XCLE_List
.B XCLE_ListAlloc
(
.B void
) ;

.PD 1
.P
.PD
Allocates a new, empty, list.

.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a new List.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListAllocBlock
.PD 0


.I XCLE_List
.B XCLE_ListAllocBlock
(
.I unsigned
.I long
.B numb
) ;

.PD 1
.P
.PD
Allocates a new, empty, list, with space reserved.

.B
	Args
.RS
.IP _ 2
.I numb
	: size of space to reservate
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a new list, with space already allocated for 'numb' objects.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListCopy
.PD 0


.I XCLE_List
.B XCLE_ListCopy
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Copies a List object and its contents.

.B
	Args
.RS
.IP _ 2
.I lst
	: List object to copy
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a complete (recursive) copy of the List.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This method copies the List given, and fills it with copies of the original's
contents. Thus, this is a complete reallocation, and no memory buffer from the
original List is reused. The new objects' reference counts are set to zero,
while the originals' are unchanged.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListClone
.PD 0


.I XCLE_List
.B XCLE_ListClone
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Clones a List object.

.B
	Args
.RS
lst
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if no memory was available
.IP "" 0
else a clone of the List object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
This method does only a "flat" copy of the List object, filling the copy with
direct references to the original's objects. The new List reference count is
set to zero, while the contained objects' reference counts are unchanged.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListDup
.PD 0


.I XCLE_List
.B XCLE_ListUpRef
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Increases the reference count of a List, and of all the objects it contains.

.B
	Args
.RS
.IP _ 2
.I lst
	: the List to reference
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument
.IP "" 0
else the original List object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListDrop
.PD 0


.I XCLE_List
.B XCLE_ListDrop
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Decreases the reference count of a List, and of all the objects it contains.

.B
	Args
.RS
.IP _ 2
.I lst
	: the List to dereference
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument
.IP "" 0
else the original List object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListFree
.PD 0


.I void
.B XCLE_ListFree
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Frees a List object, if its reference count is zero.

.B
	Args
.RS
.IP _ 2
.I lst
	: List object to free
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListLen
.PD 0


.I unsigned
.I long
.B XCLE_ListLen
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Returns the length of a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to query
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL List was passed as argument
.IP "" 0
else the List's length.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListGet
.PD 0


.I XCLE_Object
.B XCLE_ListGet
(
.I XCLE_List
.B lst
,
.I unsigned
.I long
.B pos
) ;

.PD 1
.P
.PD
Gets an object by its position in a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to query
.IP "" 0
.IP _ 2
.I pos
	: position of the object to retrieve
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL List was passed as argument, or the index was out of range
.IP "" 0
else the retrieved object.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument, or the index was out of range
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListPut
.PD 0


.I XCLE_Object
.B XCLE_ListPut
(
.I XCLE_List
.B lst
,
.I unsigned
.I long
.B pos
,
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Puts an object by its position in a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.IP _ 2
.I pos
	: position of the object to put
.IP "" 0
.IP _ 2
.I obj
	: object to put into the List
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument, or the index was out of range
.IP "" 0
else the original object that was at this position
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument, or the index was out of range
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListDel
.PD 0


.I XCLE_List
.B XCLE_ListDel
(
.I XCLE_List
.B lst
,
.I unsigned
.I long
.B beg
,
.I unsigned
.I long
.B end
) ;

.PD 1
.P
.PD
Deletes the objects in positions [beg,end[ in an List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.IP _ 2
.I beg
	: beginning of the section to cut, starting at 0
.IP "" 0
.IP _ 2
.I end
	: end of this section
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed, indexes were out of range, or a reallocation failed
.IP "" 0
else the original List.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument, or indexes were out of range
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListIns
.PD 0


.I XCLE_List
.B XCLE_ListIns
(
.I XCLE_List
.B lst
,
.I unsigned
.I long
.B pos
,
.I XCLE_List
.B lsi
) ;

.PD 1
.P
.PD
Inserts the objects of the second List in the first, between the pos-th and the pos+1-th objects.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to insert into
.IP "" 0
.IP _ 2
.I pos
	: position of insertion point, starting at 0
.IP "" 0
.IP _ 2
.I lsi
	: inserted List
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed, indexes were out of range, or a reallocation failed
.IP "" 0
else the first List, with the second's objects inserted.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument, or indexes were out of range
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
To prevent internal structures and memory corruption, 'lsi' is emptied after successfull
insertion. Using a null 'pos' means the second list is inserted at the very beginning
of 'lst', while a 'pos' equal to the first List length means to insert at the very end.
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListPush
.PD 0


.I XCLE_List
.B XCLE_ListPush
(
.I XCLE_List
.B lst
,
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Adds an object at the end of a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.IP _ 2
.I obj
	: object to append
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List or object was passed as argument, or a reallocation failed
.IP "" 0
else the updated List, with the object appended.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List or object was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListPop
.PD 0


.I XCLE_Object
.B XCLE_ListPop
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Deletes an object from the end of a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument, the List was empty, or a reallocation failed
.IP "" 0
else the object deleted from the ned of the List.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument or the List was empty
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListUnshift
.PD 0


.I XCLE_List
.B XCLE_ListUnshift
(
.I XCLE_List
.B lst
,
.I XCLE_Object
.B obj
) ;

.PD 1
.P
.PD
Adds an object at the beginning of a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.IP _ 2
.I obj
	: object to prepend
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument or a reallocation failed
.IP "" 0
else the updated list, with the object prepended.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListShift
.PD 0


.I XCLE_Object
.B XCLE_ListShift
(
.I XCLE_List
.B lst
) ;

.PD 1
.P
.PD
Deletes an object from the beginning of a List.

.B
	Args
.RS
.IP _ 2
.I lst
	: List to update
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I NULL
	: if a NULL List was passed as argument, the List was empty, or a reallocation failed
.IP "" 0
else the object deleted from the beginning of the List.
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument or the List was empty
.IP "" 0
.IP _ 2
.I ENOMEM
	: if no memory was available
.IP "" 0
.RE
.PD 1
.P
.PD





.PD 1
.P
.PD
.SS Function XCLE_ListMap
.PD 0


.I unsigned
.I long
.B XCLE_ListMap
(
.I XCLE_List
.B lst
,
.B 
,
.I void
.B *
.B data
) ;

.PD 1
.P
.PD
Executes a function on all objects of a List, totalizing the return values.

.B
	OOPS
.RS
Something was wrong here
.RE
.PD 1
.P
.PD










.PD 1
.P
.PD
.SS Function XCLE_ListSort
.PD 0


.I XCLE_List
.B XCLE_ListSort
(
.I XCLE_List
.B lst
,
.B 
) ;

.PD 1
.P
.PD
Sorts a List, using the provided object comparison function.

.B
	OOPS
.RS
Something was wrong here
.RE
.PD 1
.P
.PD










.PD 1
.P
.PD
.SS Function XCLE_ListEqual
.PD 0


.I unsigned
.I char
.B XCLE_ListEqual
(
.I XCLE_List
.B lst
,
.I XCLE_List
.B lstw
) ;

.PD 1
.P
.PD
Compares two Lists.

.B
	Args
.RS
.IP _ 2
.I lst
	: List object to compare
.IP "" 0
.IP _ 2
.I lstw
	: reference List
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I -1
	: if a NULL List was passed as argument
.IP "" 0
.IP _ 2
.I 0
	: if the two Lists were distinct
.IP "" 0
.IP _ 2
.I +1
	: if the two Lists were identical (i.e., same length, and equal objects)
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Errors
.RS
.IP _ 2
.I EINVAL
	: if a NULL List was passed as argument
.IP "" 0
.RE
.PD 1
.P
.PD








